@@ -436,7 +436,7 @@ static gboolean on_port_tested(gpointer data)
                                      _
                                      ("Port is <span font_weight=\"bold\" fgcolor=\"red\">closed</span>"));
         } else {
-            trg_error_dialog(GTK_WINDOW(data), response);
+          trg_error_dialog(GTK_WINDOW(response->cb_data), response);
         }
     }
 
