@@ -340,7 +340,7 @@ gchar *make_error_message(JsonObject * response, int status)
 {
     if (status == FAIL_JSON_DECODE) {
         return g_strdup(_("JSON decoding error."));
-    } else if (status == FAIL_RESPONSE_UNSUCCESSFUL) {
+    } else if (response && status == FAIL_RESPONSE_UNSUCCESSFUL) {
         const gchar *resultStr =
             json_object_get_string_member(response, "result");
         if (resultStr == NULL)
@@ -616,24 +616,16 @@ gboolean should_be_minimised(int argc, char *argv[])
 GtkWidget *trg_hbox_new(gboolean homogeneous, gint spacing)
 {
     GtkWidget *box;
-#if GTK_CHECK_VERSION( 3, 0, 0 )
     box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, spacing);
     gtk_box_set_homogeneous(GTK_BOX(box), homogeneous);
-#else
-    box = gtk_hbox_new(homogeneous, spacing);
-#endif
     return box;
 }
 
 GtkWidget *trg_vbox_new(gboolean homogeneous, gint spacing)
 {
     GtkWidget *box;
-#if GTK_CHECK_VERSION( 3, 0, 0 )
     box = gtk_box_new(GTK_ORIENTATION_VERTICAL, spacing);
     gtk_box_set_homogeneous(GTK_BOX(box), homogeneous);
-#else
-    box = gtk_vbox_new(homogeneous, spacing);
-#endif
     return box;
 }
 
